==> julian-0.0.1.tar.gz: lib/src/gregorian.dart <==
// Copyright (c) 2016, Christian Stewart. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'julian.dart';
import 'julian_base.dart';

/// A day in the proleptic Gregorian calendar.
class Gregorian implements CalendarDay {
  final int year, month, day;

  static const int daysIn400Years = 146097;

  /// Julian day number is 0 for Gregorian(-4713, 11, 24), i.e. 4714 BC Nov 24th.
  static const Gregorian julianDayNumberZeroDate =
      const Gregorian(-4713, 11, 24);

  static const num meanYearLengthInDays = 365.2425;

  static const Duration meanYearLengthAsDuration =
      const Duration(days: 365, hours: 5, minutes: 49, seconds: 12);

  /// A calendar day in the proleptic Gregorian calendar
  /// year ∈ ℤ i.e. {... 2, 1, 0, -1 ...} where 1 ⇔ AD 1, 0 ⇔ 1 BC, -1 ⇔ 2 BC
  /// month ∈ ℕ, i.e. {1, ..., 12}
  /// day ∈ ℕ, i.e. {1, ..., 31}
  const Gregorian(this.year, this.month, this.day);

  const Gregorian.fromBC(int year, this.month, this.day)
      : this.year = -(year - 1);

  Gregorian.fromDateTime(DateTime dt)
      : this.year = dt.year,
        this.month = dt.month,
        this.day = dt.day;

  factory Gregorian.fromJulian(Julian julian) =>
      new Gregorian.fromJulianDayNumber(julian.julianDayNumber);

  factory Gregorian.fromJulianDayNumber(int jdn) {
    // Gregorian(0, 3, 1) <==> 1721120 julian day number
    const int j0 = 1721120;
    /*
    There are 146097 days in 400 Gregorian years. The Gregorian calendar
    repeats exactly after 400 years. Thus adding a multiple of
    146097 days does not cause the day/month values to change.
     */
    final num daysDisplacement = (jdn >= j0)
        ? 0
        : ((jdn - j0).abs() / daysIn400Years).ceil() * daysIn400Years;
    // Days from the displaced date to j0
    int daysFromJ0 = jdn + daysDisplacement - j0,
        yearsFromJ0 = daysFromJ0 ~/ meanYearLengthInDays,
        y = yearsFromJ0 - (daysDisplacement ~/ meanYearLengthInDays);
    int daysFromMarch1 = daysFromJ0 -
        (yearsFromJ0 * 365 +
            yearsFromJ0 ~/ 4 -
            yearsFromJ0 ~/ 100 +
            yearsFromJ0 ~/ 400);
    // March <=> m = 0, April <=> m = 1, ...
    int m = (((daysFromMarch1 + 1 + 30.19694) / 30.6).floor() - 1);
    int d = daysFromMarch1 + 1 - (m * 30.6).round();
    m += 3;
    if (m > 12) {
      m -= 12;
      y += 1;
    }
    if (m == 2 && d > 28) {
      // note: this d can be up to 367 on leap years!
      final int febLength = isLeapYear(y) ? 29 : 28;
      final int surplus = d - febLength;
      if (surplus > 0) {
        m = 3;
        d = surplus;
      }
    }
    return new Gregorian(y, m, d);
  }

  /// Get the corresponding Julian day number
  int get julianDayNumber {
    /*
    Julian day number is 0 for Gregorian(-4713, 11, 24) (beginning at 12 noon).
    Gregorian calendar repeats exactly after 400 years.
    8000 is a multiple of 400 and also 8000 - 4713 >> 0.
    Thus 8000 is a suitable multiple for a period of years to displace the years
    of the epoch and this() to values > 0000/Mar/01 without invalidating the
    month and day values.
     */
    final int yearsDisplacement =
        8000 * (year >= 0 ? 1 : (1 + year.abs() / 8000).ceil());
    // Days from y/m/d (>= 0000/Mar/01) to 0000/Mar/01 (Gregorian)
    int daysFrom_0000_03_01(y, m, d) {
      // convert to a march year start, so that leap days are an appendage
      if (m < 3) {
        y -= 1;
        m += 12;
      }
      return (y * 365) +
          (y ~/ 4) -
          (y ~/ 100) +
          (y ~/ 400) +
          /*
          gives the number of days since March 1 and comes from the repetition
          of days in the month from March in groups of five:
          Mar–Jul: 	31 30 31 30 31
          Aug–Dec: 	31 30 31 30 31
          Jan–Feb: 	31 28
           */
          (((m * 153 + 3) ~/ 5) - 92) +
          (d - 1);
    }
    return daysFrom_0000_03_01(year + yearsDisplacement, month, day) -
        daysFrom_0000_03_01(julianDayNumberZeroDate.year + yearsDisplacement,
            julianDayNumberZeroDate.month, julianDayNumberZeroDate.day);
  }

  /// Returns true if year is a Gregorian leap year.
  static bool isLeapYear(int year) {
    final int y = year.abs();
    return (y % 400 == 0) || ((y % 4 == 0) && (y % 100 != 0));
  }

  /// Returns a DateTime with year, month, and day set to this.
  /// The hour is set to 12 by default since Julian days start at noon.
  DateTime toDateTime({int hour: 12}) => new DateTime(year, month, day, hour);

  /// Returns a Julian with the same Julian day number as this.
  Julian toJulian() => new Julian.fromJulianDayNumber(julianDayNumber);

  /// Returns a list of three integers: [year, month, day]
  List<int> toList() => [year, month, day];

  String toString() {
    final String bc = year >= 1 ? '' : ' (i.e. ${(year - 1).abs()} BC)';
    return 'Gregorian(year: $year$bc, month: $month, day: $day)';
  }
}

==> julian-0.0.2.tar.gz: lib/src/gregorian.dart <==
// Copyright (c) 2016, Christian Stewart. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

import 'julian.dart';
import 'julian_base.dart';

/// A day in the proleptic Gregorian calendar.
class Gregorian implements CalendarDay {
  /// The Gregorian year, as in astronomical year numbering
  final int year;
  final month, day;

  static const int daysIn400Years = 146097;

  /// Julian day number is 0 for Gregorian(-4713, 11, 24), i.e. 4714 BC Nov 24th.
  static const Gregorian julianDayNumberZeroDate =
      const Gregorian(-4713, 11, 24);

  static const num meanYearLengthInDays = 365.2425;

  static const Duration meanYearLengthAsDuration =
      const Duration(days: 365, hours: 5, minutes: 49, seconds: 12);

  /// A calendar day in the proleptic Gregorian calendar.
  ///
  /// [year] ∈ ℤ i.e. {... 2, 1, 0, -1 ...} where 1 ⇔ AD 1, 0 ⇔ 1 BC, -1 ⇔ 2 BC.
  /// [month] ∈ ℕ, i.e. {1, ..., 12}.
  /// [day] ∈ ℕ, i.e. {1, ..., 31}.
  const Gregorian(this.year, this.month, this.day);

  const Gregorian.fromBC(int year, this.month, this.day)
      : this.year = -(year - 1);

  Gregorian.fromDateTime(DateTime dt)
      : this.year = dt.year,
        this.month = dt.month,
        this.day = dt.day;

  factory Gregorian.fromJulian(Julian julian) =>
      new Gregorian.fromJulianDayNumber(julian.julianDayNumber);

  factory Gregorian.fromJulianDayNumber(int jdn) {
    // Gregorian(0, 3, 1) <==> 1721120 julian day number
    const int j0 = 1721120;
    /*
    There are 146097 days in 400 Gregorian years. The Gregorian calendar
    repeats exactly after 400 years. Thus adding a multiple of
    146097 days does not cause the day/month values to change.
     */
    final num daysDisplacement = (jdn >= j0)
        ? 0
        : ((jdn - j0).abs() / daysIn400Years).ceil() * daysIn400Years;
    // Days from the displaced date to j0
    int daysFromJ0 = jdn + daysDisplacement - j0,
        yearsFromJ0 = daysFromJ0 ~/ meanYearLengthInDays,
        y = yearsFromJ0 - (daysDisplacement ~/ meanYearLengthInDays);
    int daysFromMarch1 = daysFromJ0 -
        (yearsFromJ0 * 365 +
            yearsFromJ0 ~/ 4 -
            yearsFromJ0 ~/ 100 +
            yearsFromJ0 ~/ 400);
    // March <=> m = 0, April <=> m = 1, ...
    int m = (((daysFromMarch1 + 1 + 30.19694) / 30.6).floor() - 1);
    int d = daysFromMarch1 + 1 - (m * 30.6).round();
    m += 3;
    if (m > 12) {
      m -= 12;
      y += 1;
    }
    if (m == 2 && d > 28) {
      // note: this d can be up to 367 on leap years!
      final int febLength = isLeapYear(y) ? 29 : 28;
      final int surplus = d - febLength;
      if (surplus > 0) {
        m = 3;
        d = surplus;
      }
    }
    return new Gregorian(y, m, d);
  }

  /// Get the corresponding Julian day number
  int get julianDayNumber {
    /*
    Julian day number is 0 for Gregorian(-4713, 11, 24) (beginning at 12 noon).
    Gregorian calendar repeats exactly after 400 years.
    8000 is a multiple of 400 and also 8000 - 4713 >> 0.
    Thus 8000 is a suitable multiple for a period of years to displace the years
    of the epoch and this() to values > 0000/Mar/01 without invalidating the
    month and day values.
     */
    final int yearsDisplacement =
        8000 * (year >= 0 ? 1 : (1 + year.abs() / 8000).ceil());
    // Days from y/m/d (>= 0000/Mar/01) to 0000/Mar/01 (Gregorian)
    int daysFrom_0000_03_01(y, m, d) {
      // convert to a march year start, so that leap days are an appendage
      if (m < 3) {
        y -= 1;
        m += 12;
      }
      return (y * 365) +
          (y ~/ 4) -
          (y ~/ 100) +
          (y ~/ 400) +
          /*
          gives the number of days since March 1 and comes from the repetition
          of days in the month from March in groups of five:
          Mar–Jul: 	31 30 31 30 31
          Aug–Dec: 	31 30 31 30 31
          Jan–Feb: 	31 28
           */
          (((m * 153 + 3) ~/ 5) - 92) +
          (d - 1);
    }
    return daysFrom_0000_03_01(year + yearsDisplacement, month, day) -
        daysFrom_0000_03_01(julianDayNumberZeroDate.year + yearsDisplacement,
            julianDayNumberZeroDate.month, julianDayNumberZeroDate.day);
  }

  /// Returns true if year is a Gregorian leap year.
  static bool isLeapYear(int year) {
    final int y = year.abs();
    return (y % 400 == 0) || ((y % 4 == 0) && (y % 100 != 0));
  }

  /// Returns a DateTime with year, month, and day set to this.
  /// The hour is set to 12 by default since Julian days start at noon.
  DateTime toDateTime({int hour: 12}) => new DateTime(year, month, day, hour);

  /// Returns a Julian with the same Julian day number as this.
  Julian toJulian() => new Julian.fromJulianDayNumber(julianDayNumber);

  /// Returns a list of three integers: [year, month, day]
  List<int> toList() => [year, month, day];

  String toString() {
    final String bc = year >= 1 ? '' : ' (i.e. ${(year - 1).abs()} BC)';
    return 'Gregorian(year: $year$bc, month: $month, day: $day)';
  }
}
